@@ -1,4 +1,194 @@
-#' @title funBranin
+#' @title makeSpotFunList
+#'
+#' @description Generate a list of spot functions
+#'
+#' @param vector2Matrix logical. Convert vector input to matrix.
+#' Default: TRUE, so it can be used with \code{\link[stats]{optim}}.
+#'
+#' @return list of functions
+#'
+#' @importFrom stats optim
+#'
+#' @examples
+#' fr <- makeSpotFunList()
+#' optim(c(-1.2,1), fr[[1]])
+#'
+#' @export
+#'
+makeSpotFunList <- function(vector2Matrix = TRUE) {
+  funList <- list()
+  if (vector2Matrix == FALSE){
+   funList[[1]] <- funSphere
+    funList[[2]] <- funShiftedSphere
+    funList[[3]] <- funBranin
+    funList[[4]] <- funGoldsteinPrice
+    funList[[5]] <- funSoblev99
+    funList[[6]] <- funIshigami
+return(funList)
+  }
+  
+  funList[[1]] <- function(x) {
+    funSphere(x = matrix(x, 1,))
+  }
+  funList[[2]] <- function(x) {
+    funShiftedSphere(x = matrix(x, 1,),
+                     a = 1)
+  }
+  funList[[3]] <- function(x) {
+    funBranin(x = matrix(x, 1,))
+  }
+ 
+  funList[[4]] <- function(x) {
+    funGoldsteinPrice(x = matrix(x, 1,))
+  }
+  funList[[5]] <- function(x) {
+    funSoblev99(x = matrix(x, 1,))
+  }
+  funList[[6]] <- function(x) {
+    funIshigami(x = matrix(x, 1,),
+                a = 7,
+                b = 0.1)
+  }
+  return(funList)
+}
+
+#' @title makeMoreFunList
+#'
+#' @description Generate a list of benchmark functions.
+#' Based on the More(1981) paper.
+#' Contains the first 13 function from the paper.
+#' Function numbers are the same as in the paper.
+#'
+#' @param vector2Matrix logical. Convert vector input to matrix.
+#' Default: TRUE, so it can be used with \code{\link[stats]{optim}}.
+#'
+#' @return list of functions with starting points and optimum points.
+#' 
+#' @references
+#' More, J. J., Garbow, B. S., and  Hillstrom, K. E. (1981).
+#' Testing unconstrained optimization software.
+#' \emph{ACM Transactions on Mathematical Software (TOMS)}, \emph{7}(1), 17-41.
+#' \doi{10.1145/355934.355936}
+#'
+#' @importFrom stats optim
+#'
+#' @examples
+#' # Generate function list.
+#' # Here we use the default setting \code{vector2Matrix = TRUE},
+#' # so the function list can be passed to \code{\link[stats]{optim}}.
+#' 
+#' fl <- makeMoreFunList()
+#' optim(par=c(-1.2,1), fn=fl$funList[[1]])
+#' optim(par=fl$startPointList[[1]], fn=fl$funList[[1]])$value
+#' optim(par=fl$startPointList[[1]], fn=fl$funList[[1]],NULL, method = "CG", hessian = FALSE)$value
+#' optim(fl$startPointList[[1]], fl$funList[[1]],NULL, method = "BFGS", hessian = FALSE)$value
+#' optim(fl$startPointList[[1]], fl$funList[[1]],NULL, method = "L-BFGS-B", hessian = FALSE)$value
+#'
+#' @export
+#'
+makeMoreFunList <- function(vector2Matrix = TRUE) {
+  startPointList <- list( c(-1.2,1),
+                          c(0.5, -2),
+                          c(0,1),
+                          c(1,1),
+                          c(1,1),
+                          c(0.3, 0.4),
+                          c(-1,0,0),
+                          c(1,1,1),
+                          c(0.4,1,0),
+                          c(0.02,4000,250),
+                          c(5,2.5,0.15),
+                          c(0,10,20),
+                          c(3,-1,0,1)
+                          )
+  dimVec <- sapply(X=startPointList, FUN=length)
+  #c(2,2,2,2,2,2,3,3,3,3,3,3,4)
+  
+  optimPointList <- list( c(1, 1),
+                              c(5, 4),
+                              c(1.098*1e-5, 9.106),
+                              c(1e6, 2e-6),
+                              c(3, 0.5),
+                              c(0.2578, 0.2578), #m=10
+                              c(1,0,0),
+                              c(NA,NA,NA),
+                              c(NA,NA,NA),
+                              c(NA,NA,NA),
+                              c(50, 25, 1.5),
+                              c(1,10,1), # and: x_1=x_2 and x_3=0
+                              c(0, 0, 0 )
+  )
+  
+  
+  funList <- list()
+  if (vector2Matrix == FALSE){
+    funList[[1]] <- funRosen
+    funList[[2]] <- funFreudRoth
+    funList[[3]] <- funPowellBs
+    funList[[4]] <- funBrownBs
+    funList[[5]] <- funBeale
+    funList[[6]] <- funJennSamp
+    funList[[7]] <- funHelical
+    funList[[8]] <- funBard
+    funList[[9]] <- funGauss
+    funList[[10]] <- funMeyer
+    funList[[11]] <- funGulf
+    funList[[12]] <- funBox3d
+    funList[[13]] <- funPowellS
+    return(list(funList=funList,
+                dimVec=dimVec,
+                startPointList=startPointList,
+                optimPointList=optimPointList))
+  }
+  
+  funList[[1]] <- function(x) {
+    funRosen(x = matrix(x, 1,))
+  }
+  funList[[2]] <- function(x) {
+    funFreudRoth(x = matrix(x, 1,))
+  }
+  funList[[3]] <- function(x) {
+    funPowellBs(x = matrix(x, 1,))
+  }
+  funList[[4]] <- function(x) {
+    funBrownBs(x = matrix(x, 1,))
+  }
+  funList[[5]] <- function(x) {
+    funBeale(x = matrix(x, 1,))
+  }
+  funList[[6]] <- function(x) {
+    funJennSamp(x = matrix(x, 1,))
+  }
+  funList[[7]] <- function(x) {
+    funHelical(x = matrix(x, 1,))
+  }
+  funList[[8]] <- function(x) {
+    funBard(x = matrix(x, 1,))
+  }
+  funList[[9]] <- function(x) {
+    funGauss(x = matrix(x, 1,))
+  }
+  funList[[10]] <- function(x) {
+    funMeyer(x = matrix(x, 1,))
+  }
+  funList[[11]] <- function(x) {
+    funGulf(x = matrix(x, 1,))
+  }
+  funList[[12]] <- function(x) {
+    funBox3d (x = matrix(x, 1,))
+  }
+  funList[[13]] <- function(x) {
+    funPowellS(x = matrix(x, 1,))
+  }
+  return(list(funList=funList,
+              dimVec=dimVec,
+              startPointList=startPointList,
+              optimPointList=optimPointList))
+}
+
+
+
+#' @title funBranin (No. 1)
 #'
 #' @description Branin Test Function
 #'
@@ -22,9 +212,9 @@ funBranin <- function (x) {
          , 1) # number of columns
 }
 
-#' funRosen2
+#' @title  funRosen2 (No. 2a)
 #'
-#' Rosenbrock Test Function (2-dim)
+#' @description  Rosenbrock Test Function (2-dim)
 #'
 #' @param x matrix of points to evaluate with the function. Rows for points and columns for dimension.
 #'
@@ -45,9 +235,9 @@ funRosen2 <- function (x) {
          , 1) # number of columns
 }
 
-#' funRosen
+#' @title funRosen (No. 2, More No. 1)
 #'
-#' Rosenbrock Test Function
+#' @description  Rosenbrock Test Function
 #'
 #' @param x matrix of points to evaluate with the function. Rows for points and columns for dimension.
 #'
@@ -82,12 +272,12 @@ funRosen <- function (x) {
          , 1) # number of columns
 }
 
-#' funSphere
+#' @title funSphere (No. 3)
 #'
-#' Sphere Test Function
+#' @description  Sphere Test Function
 #'
 #' @param x matrix of points to evaluate with the function. Rows for points and columns for dimension.
-#' 
+#'
 #' @seealso \code{\link{funShiftedSphere}}
 #'
 #' @return 1-column matrix with resulting function values
@@ -98,7 +288,7 @@ funRosen <- function (x) {
 #'
 #' @export
 #'
-funSphere <- function (x) {
+funSphere <- function(x) {
   matrix(apply(x, # matrix
                1, # margin (apply over rows)
                function(x) {
@@ -107,15 +297,15 @@ funSphere <- function (x) {
          , 1) # number of columns
 }
 
-#' @title funShiftedSphere
+#' @title funShiftedSphere (No. 4)
 #'
 #' @description  Shifted Sphere Test Function with optimum at x_opt = a and f(x_opt) = 0
-#' 
+#'
 #' @seealso \code{\link{funSphere}}
 #'
 #' @param x matrix of points to evaluate with the function. Rows for points and columns for dimension.
-#' 
-#' @param a offset added, i.e., f = sum (x-a)^2
+#'
+#' @param a offset added, i.e., f = sum (x-a)^2. Default: \code{1}.
 #'
 #' @return 1-column matrix with resulting function values
 #'
@@ -126,19 +316,19 @@ funSphere <- function (x) {
 #'
 #' @export
 #'
-funShiftedSphere <- function (x, a) {
+funShiftedSphere <- function (x, a = 1) {
   f <- function(x, a) {
-      sum((x-a) ^ 2)  # objective function
+    sum((x - a) ^ 2)  # objective function
   }
   matrix(apply(x, # matrix
                1, # margin (apply over rows)
-              f,
-              a),
+               f,
+               a),
          , 1) # number of columns
 }
 
 
-#' @title Goldstein-Price Test Function
+#' @title Goldstein-Price Test Function (No. 5)
 #'
 #' @description  An implementation of Booker et al.’s method on
 #' a re-scaled/coded version of the 2-dim Goldstein–Price function
@@ -174,51 +364,51 @@ funGoldsteinPrice <- function (x) {
          , 1) # number of columns
 }
 
-#' @title Sobol and Levitan Test Function
+#' @title Sobol and Levitan Test Function (No. 6)
+#'
+#' @description  An implementation of the Sobol-Levitan function.
 #'
-#' @description  An implementation of the Sobol-Levitan function. 
-#' 
 #' f(x) = exp(sum b_i x_i) - I_d + c_0, where
-#'        I_d = prod( (exp(b_i) -1) / b_i) 
-#' 
-#' The value of the elements in the b-vector (b1, ..., bd) affect the importance 
+#'        I_d = prod( (exp(b_i) -1) / b_i)
+#'
+#' The value of the elements in the b-vector (b1, ..., bd) affect the importance
 #' of the corresponding x-variables.
 #' Sobol' & Levitan (1999) use two different b-vectors:
 #'   (1.5, 0.9, 0.9, 0.9, 0.9, 0.9), for d = 6, and
 #'   (0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 0.4, 0.4, 0.4, 0.4, 0.4,
-#'    0.4, 0.4, 0.4, 0.4, 0.4), for d = 20. 
+#'    0.4, 0.4, 0.4, 0.4, 0.4), for d = 20.
 #' Our implementation uses the default b vector:
-#'  b = c(0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 
-#'      0.4, 0.4, 0.4, 0.4, 0.4, 0.4, 0.4, 0.4, 0.4, 0.4) (when d<=20).    
-#'   
-#' Moon et al. (2012) scale the output to have a variance of 100. 
+#'  b = c(0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 0.6,
+#'      0.4, 0.4, 0.4, 0.4, 0.4, 0.4, 0.4, 0.4, 0.4, 0.4) (when d<=20).
+#'
+#' Moon et al. (2012) scale the output to have a variance of 100.
 #' For d = 20, they use three different b-vectors:
-#'   (2, 1.95, 1.9, 1.85, 1.8, 1.75, 1.7, 1.65, 0.4228, 0.3077, 0.2169, 0.1471, 
+#'   (2, 1.95, 1.9, 1.85, 1.8, 1.75, 1.7, 1.65, 0.4228, 0.3077, 0.2169, 0.1471,
 #'    0.0951, 0.0577, 0.0323, 0.0161, 0.0068, 0.0021, 0.0004, 0),
 #'   (1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0), and
-#'   (2.6795, 2.2289, 1.8351, 1.4938, 1.2004, 0.9507, 0.7406, 0.5659, 0.4228, 
-#'    0.3077, 0.2169, 0.1471, 0.0951, 0.0577, 0.0323, 0.0161, 0.0068, 0.0021, 0.0004, 0). 
-#' 
-#' The generally used value of c0 is c0 = 0. 
-#' The function is evaluated on xi in [0, 1], for all i = 1, ..., d. 
-#' 
-#' @references 
-#' Moon, H., Dean, A. M., & Santner, T. J. (2012). 
-#' Two-stage sensitivity-based group screening in computer experiments. 
+#'   (2.6795, 2.2289, 1.8351, 1.4938, 1.2004, 0.9507, 0.7406, 0.5659, 0.4228,
+#'    0.3077, 0.2169, 0.1471, 0.0951, 0.0577, 0.0323, 0.0161, 0.0068, 0.0021, 0.0004, 0).
+#'
+#' The generally used value of c0 is c0 = 0.
+#' The function is evaluated on xi in [0, 1], for all i = 1, ..., d.
+#'
+#' @references
+#' Moon, H., Dean, A. M., & Santner, T. J. (2012).
+#' Two-stage sensitivity-based group screening in computer experiments.
 #' Technometrics, 54(4), 376-387.
-#' 
-#' Sobol', I. M., & Levitan, Y. L. (1999). On the use of variance reducing 
-#' multipliers in Monte Carlo computations of a global sensitivity index. 
+#'
+#' Sobol', I. M., & Levitan, Y. L. (1999). On the use of variance reducing
+#' multipliers in Monte Carlo computations of a global sensitivity index.
 #' Computer Physics Communications, 117(1), 52-61.
-#' 
+#'
 #' @param x (\code{m,2})-matrix of points to evaluate with the function.
-#' Values should be >= 0 and <= 1, i.e., x_i in [0,1]. 
-#' 
+#' Values should be >= 0 and <= 1, i.e., x_i in [0,1].
+#'
 #' @param b d-dimensional vector (optional), with default value
-#'      b = c(0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 
+#'      b = c(0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 0.6, 0.6,
 #'      0.4, 0.4, 0.4, 0.4, 0.4, 0.4, 0.4, 0.4, 0.4, 0.4) (when d<=20)
-#'      
-#' @param c0 constant term (optional), with default value 0          
+#'
+#' @param c0 constant term (optional), with default value 0
 #'
 #' @return 1-column matrix with resulting function values
 #'
@@ -229,15 +419,17 @@ funGoldsteinPrice <- function (x) {
 #' @export
 #'
 funSoblev99 <- function(x,
-                        b = c(rep(0.6,10),
-                              rep(0.4,10)),
+                        b = c(rep(0.6, 10),
+                              rep(0.4, 10)),
                         c0 = 0)
 {
-  d <- length(x)
+  d <- dim(x)[2]
   db <- length(b)
   
   if (d > db) {
-    stop('Value of the d-dimensional vector b is required.')
+    print(d)
+    print(db)
+    stop('Stoped because value of the d-dimensional vector b is missing or wrong dimension for b.')
   }
   matrix(apply(x, # matrix
                1, # margin (apply over rows)
@@ -254,40 +446,40 @@ funSoblev99 <- function(x,
                    xi  <- par[ii]
                    sum <- sum + bi * xi
                  }
-                exp(sum) - Id + c0
+                 exp(sum) - Id + c0
                }),
          , 1) # number of columns
 }
 
-#' @title Ishigami Test Function
+#' @title Ishigami Test Function (No. 7)
+#'
+#' @description  An implementation of the 3-dim Ishigami function.
 #'
-#' @description  An implementation of the 3-dim Ishigami function. 
-#' 
 #' f(x) = sin(x_1) + a sin^2(x_2) + b x_3^4sin(x_1)
-#' 
-#' The Ishigami function of Ishigami & Homma (1990) is used as an example for 
-#' uncertainty and sensitivity analysis methods, 
-#' because it exhibits strong nonlinearity and nonmonotonicity. 
-#' It also has a peculiar dependence on x_3, as described by Sobol' & Levitan (1999). 
-#' The independent distributions of the input random variables are usually: 
+#'
+#' The Ishigami function of Ishigami & Homma (1990) is used as an example for
+#' uncertainty and sensitivity analysis methods,
+#' because it exhibits strong nonlinearity and nonmonotonicity.
+#' It also has a peculiar dependence on x_3, as described by Sobol' & Levitan (1999).
+#' The independent distributions of the input random variables are usually:
 #' x_i ~ Uniform[-pi, pi ], for all i = 1, 2, 3.
-#' 
-#' @references 
-#' Ishigami, T., & Homma, T. (1990, December). 
-#' An importance quantification technique in uncertainty analysis for computer models. 
-#' In Uncertainty Modeling and Analysis, 1990. Proceedings., 
+#'
+#' @references
+#' Ishigami, T., & Homma, T. (1990, December).
+#' An importance quantification technique in uncertainty analysis for computer models.
+#' In Uncertainty Modeling and Analysis, 1990. Proceedings.,
 #' First International Symposium on (pp. 398-403). IEEE.
-#' 
-#' Sobol', I. M., & Levitan, Y. L. (1999). On the use of variance reducing 
-#' multipliers in Monte Carlo computations of a global sensitivity index. 
+#'
+#' Sobol', I. M., & Levitan, Y. L. (1999). On the use of variance reducing
+#' multipliers in Monte Carlo computations of a global sensitivity index.
 #' Computer Physics Communications, 117(1), 52-61.
-#' 
-#' @param x (\code{m,2})-matrix of points to evaluate with the function.
-#' Values should be >= 0 and <= 1, i.e., x_i in [0,1]. 
-#' 
+#'
+#' @param x (\code{m,3})-matrix of points to evaluate with the function.
+#' Values should be >= -pi and <= pi, i.e., x_i in [-pi,pi].
+#'
 #' @param a coefficient (optional), with default value 7
-#' @param b coefficient (optional), with default value 0.1  
-#' 
+#' @param b coefficient (optional), with default value 0.1
+#'
 #' @return 1-column matrix with resulting function values
 #'
 #' @examples
@@ -306,27 +498,27 @@ funIshigami <- function(x,
                  x1 <- x[1]
                  x2 <- x[2]
                  x3 <- x[3]
-                 sin(x1) + a * (sin(x2))^2 +  b * x3^4 * sin(x1)
+                 sin(x1) + a * (sin(x2)) ^ 2 +  b * x3 ^ 4 * sin(x1)
                }),
          , 1) # number of columns
 }
 
-#' funFreundRoth
+#' @title funFreudRoth (No. 8, More No. 2)
 #'
-#' Freundenstein and Roth Test Function 
+#' @description  2-dim Freudenstein and Roth Test Function
 #'
-#' @param x matrix of points to evaluate with the function. 
+#' @param x matrix of points to evaluate with the function.
 #' Rows for points and columns for dimension.
 #'
 #' @return 1-column matrix with resulting function values
-#' 
+#'
 #' @references
 #' More, J. J., Garbow, B. S., and  Hillstrom, K. E. (1981).
 #' Testing unconstrained optimization software.
 #' \emph{ACM Transactions on Mathematical Software (TOMS)}, \emph{7}(1), 17-41.
 #' \doi{10.1145/355934.355936}
 #'
-#' B. Freudstein, F., and Roth, B. (Oct. 1963).
+#' B. Freudenstein, F., and Roth, B. (Oct. 1963).
 #' Numerical solutions of systems of nonlinear equations.
 #' \emph{The ACM Journal}, \emph{3}(3), 550-556.
 #' \url{https://dl.acm.org/doi/10.1145/321186.321200}
@@ -334,13 +526,13 @@ funIshigami <- function(x,
 #' @examples
 #' x1 <- matrix(c(1,1),1,)
 #' funFreudRoth(x1)
-#' 
+#'
 #' # Running SPOT with 20 function evaluations with default configurations
 #' res <- spot(,funFreudRoth,c(0,0),c(10,10),control=list(funEvals=20))
 #' plotModel(res$model)
 #'
 #' @export
-#' 
+#'
 funFreudRoth <- function (x) {
   freudRoth <- function(par) {
     x <- par[1]
@@ -357,15 +549,15 @@ funFreudRoth <- function (x) {
          , 1) # number of columns
 }
 
-#' funPowellBs
+#' @title funPowellBs (No. 9, More No. 3)
 #'
-#' Powell Badly Scaled Test Function
+#' @description  2-dim Powell Badly Scaled Test Function
 #'
-#' @param x matrix of points to evaluate with the function. 
+#' @param x matrix of points to evaluate with the function.
 #' Rows for points and columns for dimension.
 #'
 #' @return 1-column matrix with resulting function values
-#' 
+#'
 #' @references
 #' More, J. J., Garbow, B. S., and  Hillstrom, K. E. (1981).
 #' Testing unconstrained optimization software.
@@ -375,19 +567,19 @@ funFreudRoth <- function (x) {
 #' Powell, M.J.D. (1970).
 #' A hybrid method for nonlinear equations.
 #' In Numerical methods for Nonlinear Algebraic Equations,
-#' P. Rabinowitz (Ed), 
+#' P. Rabinowitz (Ed),
 #' \emph{Gordon & Breach, New York.}, \emph{3}(3), 87-114.
 #'
 #' @examples
 #' x1 <- matrix(c(-1,1),1,)
 #' funPowellBs(x1)
-#' 
+#'
 #' # Running SPOT with 20 function evaluations with default configurations
 #' res <- spot(,fun=funPowellBs,c(-10,-10),c(10,10),control=list(funEvals=20))
 #' plotModel(res$model, points = rbind(c(res$xbest[1], res$xbest[2]),c(1.098e-5,9.106)))
-#' 
+#'
 #' @export
-#' 
+#'
 
 funPowellBs <- function (x) {
   powellBs <- function(par) {
@@ -405,35 +597,35 @@ funPowellBs <- function (x) {
          , 1) # number of columns
 }
 
-#' funbrownBs
+#' @title funbrownBs (No. 10, More No. 4)
 #'
-#' Brown badly scaled Test Function
-#' 
-#' @details 
+#' @description  2-dim Brown badly scaled Test Function
+#'
+#' @details
 #' n=2, m=3
 #' x0 = (1,1)
 #' f=0 at (1e6, 2e-6)
 #'
-#' @param x matrix of points to evaluate with the function. 
+#' @param x matrix of points to evaluate with the function.
 #' Rows for points and columns for dimension.
 #'
 #' @return 1-column matrix with resulting function values
-#' 
+#'
 #' @references
 #' More, J. J., Garbow, B. S., and  Hillstrom, K. E. (1981).
 #' Testing unconstrained optimization software.
 #' \url{https://www.osti.gov/servlets/purl/6650344}
-#' 
-#' 
+#'
+#'
 #' @examples
 #' x1 <- matrix(c(1,1),1,)
 #' funBrownBs(x1)
-#' 
+#'
 #' res <- spot(,fun=funBrownBs,c(-10,-10),c(10,10),control=list(funEvals=20))
 #' plotModel(res$model, points = rbind(c(res$xbest[1], res$xbest[2]),c(1.098e-5,9.106)))
 #'
 #' @export
-#' 
+#'
 funBrownBs <- function (x) {
   brownBs <- function(par) {
     x <- par[1]
@@ -450,18 +642,18 @@ funBrownBs <- function (x) {
          , 1) # number of columns
 }
 
-#' funBeale
+#' @title funBeale (No.11, More No. 5)
 #'
-#' Beale Test Function
+#' @description  2-dim Beale Test Function
 #'
-#' @param x matrix of points to evaluate with the function. 
+#' @param x matrix of points to evaluate with the function.
 #' Rows for points and columns for dimension.
 #'
 #' @return 1-column matrix with resulting function values
-#' 
+#'
 #' @references
 #' Beale, E.M.L. On an interactive method of finding a local minimum of a function of more than
-#' one variable. Tech. Rep. No. 25, Statistical Techniques Research Group, Princeton Univ., 
+#' one variable. Tech. Rep. No. 25, Statistical Techniques Research Group, Princeton Univ.,
 #' Princeton, N.J., 1958.
 #'
 #' Rosenbrock, H. (1960).
@@ -472,12 +664,12 @@ funBrownBs <- function (x) {
 #' @examples
 #' x1 <- matrix(c(1,1),1,)
 #' funBeale(x1)
-#' 
+#'
 #' res <- spot(,funBeale,c(1,-1),c(5,2),control=list(funEvals=15))
 #' plotModel(res$model)
 #'
 #' @export
-#' 
+#'
 funBeale <- function (x) {
   beale <- function(par) {
     x <- par[1]
@@ -497,15 +689,15 @@ funBeale <- function (x) {
          , 1) # number of columns
 }
 
-#' funjennSamp
+#' @title funJennSamp (No. 12, More No 6)
 #'
-#' Jennrich and Sampson Function Test Function
+#' @description 2-dim Jennrich and Sampson Function Test Function
 #'
-#' @param x matrix of points to evaluate with the function. 
+#' @param x matrix of points to evaluate with the function.
 #' Rows for points and columns for dimension.
 #'
 #' @return 1-column matrix with resulting function values
-#' 
+#'
 #' @references
 #' More, J. J., Garbow, B. S., & Hillstrom, K. E. (1981).
 #' Testing unconstrained optimization software.
@@ -520,16 +712,17 @@ funBeale <- function (x) {
 #' @examples
 #' x1 <- matrix(c(1,1),1,)
 #' funJennSamp(x1)
-#' 
+#'
 #' res <- spot(,funJennSamp,c(0,0),c(0.3,0.3))
 #' plotModel(res$model)
 #'
 #' @export
-#' 
+#'
 funJennSamp <- function (x) {
   jennSamp <- function(par, ...) {
-    if(!exists("m")){
-      m = 10}
+    if (!exists("m")) {
+      m = 10
+    }
     if (m < 2) {
       stop("Jennrich-Sampson: m must be >= 2")
     }
@@ -550,29 +743,29 @@ funJennSamp <- function (x) {
 }
 
 
-#' funHelical
+#' @title funHelical (No. 13, More No. 7)
 #'
-#' Helical Test Function
+#' @description  3-dim Helical Test Function
 #'
-#' @param x matrix of points to evaluate with the function. 
+#' @param x matrix (n x 3)-dim of points to evaluate with the function.
 #' Rows for points and columns for dimension.
 #'
 #' @return 1-column matrix with resulting function values
-#' 
+#'
 #' @references
 #' More', J. J., Garbow, B. S., and  Hillstrom, K. E. (1981).
 #' Testing unconstrained optimization software.
 #' \emph{ACM Transactions on Mathematical Software (TOMS)}, \emph{7}(1), 17-41.
 #' \doi{10.1145/355934.355936}
 #'
-#' Fletcher, R., and  Powell, M. J. (1963). 
-#' A rapidly convergent descent method for minimization. 
-#' \emph{The Computer Journal}, \emph{6}(2), 163-168. 
+#' Fletcher, R., and  Powell, M. J. (1963).
+#' A rapidly convergent descent method for minimization.
+#' \emph{The Computer Journal}, \emph{6}(2), 163-168.
 #' \doi{10.1093/comjnl/6.2.163}
 #'
 #' @examples
-#' x1 <- matrix(c(1,1),1,)
-#' funHelical(x1) 
+#' x1 <- matrix(c(1,1,1),1,)
+#' funHelical(x1)
 #' res <- spot(,funHelical,c(-40,-40,-40),c(40,40,40),control=list(funEvals=20))
 #' plotModel(res$model,which=c(1,2),type="persp",border="NA")
 #' plotModel(res$model,which=c(2,3),type="persp",border="NA")
@@ -582,7 +775,7 @@ funJennSamp <- function (x) {
 #' plotModel(res$model, which=c(2,3))
 #'
 #' @export
-#' 
+#'
 
 funHelical <- function(x) {
   helical <- function(par) {
@@ -602,33 +795,33 @@ funHelical <- function(x) {
     f2 <- 10 * (sqrt(x * x + y * y) - 1)
     f3 <- z
     f1 * f1 + f2 * f2 + f3 * f3
-  }  
+  }
   matrix(apply(x, # matrix
                1, # margin (apply over rows)
                helical),
          , 1) # number of columns
 }
 
-#' funBard
+#' @title funBard (No. 14, More No. 8)
+#'
+#' @description  3-dim Bard Test Function
 #'
-#' The Bard Test Function
-#' 
 #' x0 = (1,1,1)
 #' f = 8.21487...1e-3
 #' f = 17.4286... at (0.8406..., -infty, -infty)
 #'
-#' @param x matrix of points to evaluate with the function. 
+#' @param x matrix of points to evaluate with the function.
 #' Rows for points and columns for dimension.
 #'
 #' @return 1-column matrix with resulting function values
-#' 
+#'
 #' @references
 #' More, J. J., Garbow, B. S., and  Hillstrom, K. E. (1981).
 #' Testing unconstrained optimization software.
 #' \emph{ACM Transactions on Mathematical Software (TOMS)}, \emph{7}(1), 17-41.
 #' \doi{10.1145/355934.355936}
-#' 
-#' BARD, Y. Comparison of gradient methods for the solution of nonlinear parameter estimation 
+#'
+#' BARD, Y. Comparison of gradient methods for the solution of nonlinear parameter estimation
 #' problems SIAM J. Numer. Anal. 7 (1970), 157-186.
 #'
 #'
@@ -637,11 +830,24 @@ funHelical <- function(x) {
 #' funBard(x1)
 #'
 #' @export
-#' 
+#'
 funBard <- function (x) {
   bard <- function(par) {
-    y <- c(0.14, 0.18, 0.22, 0.25, 0.29, 0.32, 0.35, 0.39, 0.37, 0.58,
-           0.73, 0.96, 1.34, 2.10, 4.39)
+    y <- c(0.14,
+           0.18,
+           0.22,
+           0.25,
+           0.29,
+           0.32,
+           0.35,
+           0.39,
+           0.37,
+           0.58,
+           0.73,
+           0.96,
+           1.34,
+           2.10,
+           4.39)
     m <- 15
     x1 <- par[1]
     x2 <- par[2]
@@ -662,15 +868,15 @@ funBard <- function (x) {
          , 1) # number of columns
 }
 
-#' funGauss
+#' @title funGauss (No. 15, More No. 9)
 #'
-#' Gaussian Test Function
+#' @description  3-dim Gaussian Test Function
 #'
-#' @param x matrix of points to evaluate with the function. 
+#' @param x matrix of points to evaluate with the function.
 #' Rows for points and columns for dimension.
 #'
 #' @return 1-column matrix with resulting function values
-#' 
+#'
 #' @references
 #' Unpublished
 #'
@@ -678,18 +884,34 @@ funBard <- function (x) {
 #' x1 <- matrix(c(1,1,1),1,)
 #' funGauss(x1)
 #'
-#' res1 <- spot(,funGauss, 
+#' res1 <- spot(,funGauss,
 #'   c(-0.001,-0.007,-0.003),
 #'   c(0.5,1.0,1.1),
 #'   control=list(funEvals=15))
 #'   plotModel(res1$model, which = 1:2)
 #'
 #' @export
-#' 
+#'
 funGauss <- function (x) {
   gauss <- function(par) {
-    y <- c(0.0009, 0.0044, 0.0175, 0.0540, 0.1295, 0.2420, 0.3521, 0.3989,
-           0.3521, 0.2420, 0.1295, 0.0540, 0.0175, 0.0044, 0.0009)
+    y <-
+      c(
+        0.0009,
+        0.0044,
+        0.0175,
+        0.0540,
+        0.1295,
+        0.2420,
+        0.3521,
+        0.3989,
+        0.3521,
+        0.2420,
+        0.1295,
+        0.0540,
+        0.0175,
+        0.0044,
+        0.0009
+      )
     m <- 15
     x1 <- par[1]
     x2 <- par[2]
@@ -710,15 +932,15 @@ funGauss <- function (x) {
 }
 
 
-#' funMeyer
+#' @title funMeyer (No. 16, More No. 10)
 #'
-#' Meyer Test Function
+#' @description  Meyer 3-dim Test Function
 #'
-#' @param x matrix of points to evaluate with the function. 
+#' @param x matrix (dim 1x3) of points to evaluate with the function.
 #' Rows for points and columns for dimension.
 #'
 #' @return 1-column matrix with resulting function values
-#' 
+#'
 #' @references
 #' More, J. J., Garbow, B. S., and Hillstrom, K. E. (1981).
 #' Testing unconstrained optimization software.
@@ -729,7 +951,7 @@ funGauss <- function (x) {
 #' @examples
 #' x1 <- matrix(c(1,1,1),1,)
 #' funMeyer(x1)
-#' 
+#'
 #' set.seed(13)
 #' resMeyer <- spot(matrix(c(0.02,4000,250),1,3),
 #'   funMeyer,c(0,1000,200),c(3,8000,500),
@@ -741,13 +963,28 @@ funGauss <- function (x) {
 #' plotModel(resMeyer$model,which=2:3)
 #'
 #' @export
-#' 
+#'
 funMeyer <- function (x) {
-  
   meyer <- function(par) {
-    
-    y <- c(34780, 28610, 23650, 19630, 16370, 13720, 11540, 9744, 8261, 7030,
-           6005, 5147, 4427, 3820, 3307, 2872)
+    y <-
+      c(
+        34780,
+        28610,
+        23650,
+        19630,
+        16370,
+        13720,
+        11540,
+        9744,
+        8261,
+        7030,
+        6005,
+        5147,
+        4427,
+        3820,
+        3307,
+        2872
+      )
     m <- 16
     x1 <- par[1]
     x2 <- par[2]
@@ -757,25 +994,26 @@ funMeyer <- function (x) {
     
     return(sum(fi * fi))
     
-  }  
+  }
   matrix(apply(x, # matrix
                1, # margin (apply over rows)
                meyer),
          , 1) # number of columns
 }
 
-#' funGulf
+#' @title funGulf (No.17, More No. 11)
 #'
-#' Gulf research and development Test Function
+#' @description 3-dim Gulf research and development Test Function
 #'
-#' @param x matrix of points to evaluate with the function. 
+#' @param x matrix (n x 3) of points to evaluate with the function. 
 #' Rows for points and columns for dimension.
+#' Values should be larger than 0.
 #'
-#' @param ... additional parameters. 
+#' @param m  additional parameter: .
 #' The Gulf function supports an additional parameter m in the range from 3 to 100
 #'
 #' @return 1-column matrix with resulting function values
-#' 
+#'
 #' @references
 #' More, J. J., Garbow, B. S., and  Hillstrom, K. E. (1981).
 #' Testing unconstrained optimization software.
@@ -787,13 +1025,13 @@ funMeyer <- function (x) {
 #' funGulf(x1)
 #'
 #' funGulf(x1,m=50)
-#' 
+#'
 #' resGulf <- spot(,funGulf,c(0,0,0),c(100,50,5))
 #' resGulf$xbest
 #' resGulf$ybest
 #' plotModel(resGulf$model, which=1:2)
 #' plotModel(resGulf$model, which=2:3)
-#' 
+#'
 #' # x0 is an optional start point (or set of start points), specified as a matrix.
 #' # One row for each point, and one column for each optimized parameter.
 #' x0 = matrix(c(5,2.5,0.15),1,3)
@@ -802,25 +1040,16 @@ funMeyer <- function (x) {
 #' resGulf$ybest
 #'
 #' @export
-#' 
-funGulf <- function (x,...) { 
-  gulf <- function(par,...) {      
-    # convert ellipsis to list
-    args <- list(...)
-    # check m parameter in ellipsis
-    if ('m' %in% names(args)) m = args$m
-    else m=99
-    
-    #         if(!exists("m")){ m = 99} # Unnecessary, same functionality in the ellipsis handler
-    
+#'
+funGulf <- function (x, m = 99) {
+  gulf <- function(par, m) {
     ## m can be between 3 and 100
     if (m < 3 || m > 100) {
       stop("Gulf research and development function: m must be between 3 and 100")
     }
-    
     #It looks like a copy paste from the previous function.
     #       y <- c(34780, 28610, 23650, 19630, 16370, 13720, 11540, 9744, 8261, 7030,
-    #              6005, 5147, 4427, 3820, 3307, 2872) 
+    #              6005, 5147, 4427, 3820, 3307, 2872)
     p66 <- 2 / 3
     
     x1 <- par[1]
@@ -830,25 +1059,25 @@ funGulf <- function (x,...) {
     ti <- 1:m * 0.01
     y <- 25 + (-50 * log(ti)) ^ p66
     fi <- exp(-(abs(x2 - y) ^ x3) / x1) - ti
-    return( sum(fi * fi))
-  }       
+    return(sum(fi * fi))
+  }
   
   
   matrix(apply(x, # matrix
                1, # margin (apply over rows)
-               gulf,...),
+               gulf, m),
          , 1) # number of columns
 }
 
-#' funbox3D
+#' @title funbox3D (No. 18, More No. 12)
 #'
-#' Box three - dimensional Test Function
+#' @description Box three-dimensional Test Function
 #'
-#' @param x matrix of points to evaluate with the function. 
+#' @param x matrix of points to evaluate with the function.
 #' Rows for points and columns for dimension.
 #'
 #' @return 1-column matrix with resulting function values
-#' 
+#'
 #' @references
 #' More, J. J., Garbow, B. S., and  Hillstrom, K. E. (1981).
 #' Testing unconstrained optimization software.
@@ -857,14 +1086,14 @@ funGulf <- function (x,...) {
 #'
 #' Box three - dimensional, (1966).
 #' A comparison of several current optimization methods, and the use of transformations
-#'  in constrained problems. 
+#'  in constrained problems.
 #' \emph{The Computer Journal}, \emph{3}(3), 66-77.
 #' \url{https://academic.oup.com/comjnl/article/9/1/67/348150}
-#' 
+#'
 #'  @examples
 #' x <- matrix(c(1,10,1),1,)
 #' funBox3d(x)
-#' 
+#'
 #' res <- spot(,funBox3d,c(5,15,-5),c(15,5,5),control=list(funEvals=20))
 #' # plotting the graphs
 #' plotModel(res$model,which=1:2)
@@ -872,11 +1101,12 @@ funGulf <- function (x,...) {
 #' plotModel(res$model,which=c(1,3))
 #'
 #' @export
-#' 
+#'
 funBox3d <- function (x) {
   box3d <- function(par, ...) {
-    if(!exists("m")){
-      m = 20}
+    if (!exists("m")) {
+      m = 20
+    }
     if (m < 3) {
       stop("box3d: m must be >= 3")
     }
@@ -888,7 +1118,8 @@ funBox3d <- function (x) {
     fsum <- 0
     for (i in 1:m) {
       ti <- 0.1 * i
-      fi <- exp(-ti * x1) - exp(-ti * x2) - x3 * (exp(-ti) - exp(-i))
+      fi <-
+        exp(-ti * x1) - exp(-ti * x2) - x3 * (exp(-ti) - exp(-i))
       fsum <- fsum + fi * fi
     }
     return(fsum)
@@ -901,15 +1132,15 @@ funBox3d <- function (x) {
 }
 
 
-#' funpowellS
+#' @title funPowellS (No. 19, More No. 13)
 #'
-#' Powells Test Function
+#' @description  Powells 4-dim Test Function
 #'
-#' @param x matrix of points to evaluate with the function. 
+#' @param x matrix (dim 1x4) of points to evaluate with the function.
 #' Rows for points and columns for dimension.
 #'
 #' @return 1-column matrix with resulting function values
-#' 
+#'
 #' @references
 #' More, J. J., Garbow, B. S., and  Hillstrom, K. E. (1981).
 #' Testing unconstrained optimization software.
@@ -920,7 +1151,7 @@ funBox3d <- function (x) {
 #' \url{http://owos.gm.fh-koeln.de:8055/bartz/optimization-ait-master-2020/blob/master/Jupyter.d/Exercise-VIIa.ipynb}
 #' \url{http://bab10.bartzandbartz.de:8033/bartzbeielstein/bab-optimization-ait-master-2020/-/blob/master/Jupyter.d/01spotNutshell.ipynb}
 #' \url{https://www.mat.univie.ac.at/~neum/glopt/bounds.html}
-#' 
+#'
 #'
 #' Powells Test function, M. J. D. Powell, 1962
 #' An automatic method for finding the local minimum of a function.
@@ -930,17 +1161,17 @@ funBox3d <- function (x) {
 #' @examples
 #' x1 <- matrix(c(0,0,0,0),1,)
 #' funPowellS(x1)
-#' x2 <- matrix(c(3,-1,0,1),1,) 
+#' x2 <- matrix(c(3,-1,0,1),1,)
 #' funPowellS(x2)
 #' x3 <- matrix(c(0,0,0,-2),1,)
 #' funPowellS(x3)
 #' # optimization run with SPOT and 15 evaluations
-#' res_fun <- spot(,funPowellS,c(-4,-4,-4,-4 ),c(5,5,5,5),control=list(funEvals=15)) 
+#' res_fun <- spot(,funPowellS,c(-4,-4,-4,-4 ),c(5,5,5,5),control=list(funEvals=15))
 #' res_fun
 #'
 #' @export
-#' 
-funPowellS <- function(x){
+#'
+funPowellS <- function(x) {
   powellS <- function(par) {
     x1 <- par[1]
     x2 <- par[2]
@@ -958,5 +1189,5 @@ funPowellS <- function(x){
   matrix(apply(x, # matrix
                1, # margin apply over rows
                powellS),
-         ,1) # number of columns
+         , 1) # number of columns
 }
